# Generated by Django 3.2.6 on 2021-08-16 18:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reversion', '0001_squashed_0004_auto_20160611_1202'),
    ]

    operations = [
        migrations.AddIndex(
            model_name='version',
            index=models.Index(fields=['content_type', 'db'], name='reversion_v_content_f95daf_idx'),
        ),
    ]
